<?php

namespace Drupal\smtp_mail_auth_handler\Plugin\Mail;

use Drupal\Component\Serialization\Json;
use Drupal\key\Entity\Key;
use Drupal\smtp\Plugin\Mail\SMTPMailSystem;

/**
 * Defines the custom SMTP mail system for webforms with custom credentials.
 *
 * @Mail(
 *   id = "smtp_webform_mail_system",
 *   label = @Translation("SMTP Webform Mail System"),
 *   description = @Translation("SMTP mail system that supports custom credentials per webform.")
 * )
 */
class SmtpWebformMailSystem extends SMTPMailSystem {

  /**
   * The custom SMTP credentials.
   *
   * @var array
   */
  protected $customCredentials = [];

  /**
   * {@inheritdoc}
   */
  public function mail(array $message) {
    // Check if custom SMTP credentials are provided in the message parameters.
    if (isset($message['params']['smtp_credentials_key'])) {
      $key = Key::load($message['params']['smtp_credentials_key']);
      if ($key) {
        $this->customCredentials = Json::decode($key->getKeyValue());
      }
    }

    return parent::mail($message);
  }

  /**
   * Get (and, if applicable, build) the PHPMailer object.
   *
   * @return \PHPMailer\PHPMailer\PHPMailer
   *   The PHPMailer object.
   */
  protected function getMailer() {
    $mailer = parent::getMailer();

    // Inject custom credentials if they exist.
    if (!empty($this->customCredentials)) {
      $mailer->SMTPAuth = TRUE;
      $mailer->Username = $this->customCredentials['username'];
      $mailer->Password = $this->customCredentials['password'];
    }

    return $mailer;
  }

}
